// elevatorinterface.h
//	Abstract interface to set of elevators. For use by elevator threads.
//
//  DO NOT CHANGE -- part of the machine emulation
//
// Copyright (c) 1992-1996 The Regents of the University of California.
// All rights reserved.  See copyright.h for copyright notice and limitation 
// of liability and disclaimer of warranty provisions.

#ifndef ELEVATORINTERFACE_H
#define ELEVATORINTERFACE_H

#include "copyright.h"
#include "utility.h"
#include "callback.h"
#include "elevatortypes.h"

class ElevatorInterface : public CallBackObj {
public:
  // return the number of floors in the building
  virtual int getNumFloors() = 0;
  // return the number of elevators in the building
  virtual int getNumElevators() = 0;
  // determine whether it's time to terminate
  bool isDone() { return done; }
  
  // Say what object to call when an event occurs that elevator threads
  // would be interested in
  void SetCallBack(CallBackObj* callback) {
    this->callback = callback; }
  // Called by the thread that created the interface to indicate that
  // no further requests will be given, so the controller should terminate.
  void Terminate() {
    done = TRUE; CallBack(); }
  
  // Open the elevator doors; elevator must not be in motion!
  virtual void OpenDoors(int elevator) = 0;

  // Close the elevator doors.
  virtual void CloseDoors(int elevator) = 0;

  // Set the elevator into motion. Elevator doors must be closed. If elevator
  // is already in motion, this causes the elevator to stop at a new floor
  // (e.g., if a rider presses a new floor while the elevator is moving)
  virtual void MoveTo(int goingToFloor, int elevator) = 0;

  // return the current position of the elevator (which floor its on)
  virtual int WhereIsElevator(int elevator) = 0;

  // change the display to show riders where this elevator is headed
  virtual void MarkDirection(int elevator, Direction dir) = 0;

  // when the elevator device calls back, this will tell you what event(s)
  // (relevant to controller threads) triggered the callback.
  virtual ElevatorEvent getNextEvent(int *floor, int *elevator) = 0;

  // called by the bank when an elevator event occurs, or if the done flag has
  // been set; if the user callback is non-NULL, call it
  void CallBack() {
    if (callback != NULL) callback->CallBack(); }

  virtual ~ElevatorInterface() {}
  
protected:
  ElevatorInterface() { callback = NULL; done = FALSE; }
  
  CallBackObj* callback;
  bool done;
};

#endif // ELEVATORINTERFACE_H
