// riderinterface.h
//	Abstract interface to set of riders. For use by rider threads.
//
//  DO NOT CHANGE -- part of the machine emulation
//
// Copyright (c) 1992-1996 The Regents of the University of California.
// All rights reserved.  See copyright.h for copyright notice and limitation 
// of liability and disclaimer of warranty provisions.

#ifndef RIDERINTERFACE_H
#define RIDERINTERFACE_H

#include "copyright.h"
#include "utility.h"
#include "callback.h"
#include "elevatortypes.h"

// methods are roughly in the order that they are called by a rider thread

class RiderInterface : public CallBackObj {
public:
  // return the number of floors in the building
  virtual int getNumFloors() = 0;
  // return the number of elevators in the building
  virtual int getNumElevators() = 0;
  
  // Say what object to call when an event occurs that rider threads would
  // be interested in
  virtual void SetCallBack(CallBackObj* callback) {
    this->callback = callback; }
  
  // Press button outside elevator to indicate where rider wants to go
  virtual void PressButton(int onFloor, Direction goingTo) = 0;
    
  // return the indicated direction (set by MarkDirection()) of this elevator
  virtual Direction getDirection(int elevator) = 0;

  // Step onto the elevator; elevator must have < MaxRiders people on it
  // currently. Returns false if doors have closed (might have happened
  // between when riders got OpenDoors event and when they decided to step
  // on to the elevator), or if the elevator is full.
  virtual bool EnterElevator(int onFloor, int elevator) = 0;

  // Press button inside elevator to indicate where rider wants to go; thread
  // must be on the elevator
  virtual void PressFloor(int goingToFloor, int onElevator) = 0;

  // Step off the elevator. Returns false if doors have closed.
  virtual bool ExitElevator(int onFloor, int elevator) = 0;
    
  // when the elevator device calls back, this will tell you what event(s)
  // (relevant to rider threads) triggered the callback.
  virtual ElevatorEvent getNextEvent(int *floor, int *elevator) = 0;

  // called by the bank when a rider event occurs. if the user callback is
  // non-NULL, call it.
  void CallBack() {
    if (callback != NULL) callback->CallBack(); }

  virtual ~RiderInterface() {}
  
protected:
  RiderInterface() { callback = NULL; }
  
  CallBackObj* callback;
};

#endif // RIDERINTERFACE_H
