#ifndef TESTSET_H
#define TESTSET_H

#include "utility.h"
#include "debug.h"
#include <stdarg.h>

class Thread;
class TranslationEntry;
class AutoGrader;

class TestCase {
public:
  virtual void TestProc(int argc, char** argv) {}
  virtual void ReadyThread(Thread* thread) {}
  virtual void RunningThread(Thread* thread) {}
  virtual void FinishingThread(Thread* thread) {}
  virtual void TLBUpdate(int i, TranslationEntry* entry) {}
  virtual void PageFault(unsigned vaddr) {}
  virtual void Tick(long long when) {}
  virtual void Interrupt(int kind, long long when) {}
  virtual void ConfirmTimerConfiguration(bool random) {}
  virtual bool ExceptionHandler(int which) { return TRUE; }
  virtual void ConfirmNetworkReliability(double reliability) {}

#ifdef AUTOGRADING
#define MAXTESTCASES	100
protected:
  void Print(char* format...);

  static TestCase* testcases[MAXTESTCASES];

  friend class AutoGrader;
#endif  
};

#define CASEBEGIN(n)	class TestCase##n : public TestCase
#define CASESTART(n)	public: TestCase##n() { testcases[n]=this; }
#define CASEEND(n)	; static TestCase##n CASEENDh(__LINE__);
#define CASEENDh(line)	CASEENDh2(line)
#define CASEENDh2(line)	t##line

#endif
