#include "stdio.h"
#include "stdlib.h"

int fgetc(FILE stream)
{
  unsigned char c;

  while (read(stream, &c, 1)==0);

  return c;
}

// writes one character to the file f (opened file)
void fputc(char c, FILE stream)
{
  write(stream, &c, 1);
}

void fputs(const char *s, FILE stream)
{
  write(stream, (char*) s, strlen(s));
}

static char digittoascii(unsigned n, int uppercase)
{
  ASSERT(n<36);
  
  if (n<=9)
    return '0'+n;
  else if (uppercase)
    return 'A'+n-10;
  else
    return 'a'+n-10;
}

static void charprint(FILE stream, char **s, char c)
{
  ASSERT(stream == -1 || *s == NULL);
  ASSERT(stream != -1 || *s != NULL);

  if (*s != NULL)
    *((*s)++) = c;
  else
    fputc(c, stream);
}

static void integerprint(FILE stream, char **s, int n, unsigned base,
			 int min, int zpad, int upper)
{
  char buf[32];
  int i=0, digit;

  ASSERT(base>=2 && base < 36);

  if (min>32)
    min=32;

  if (n==0) {
    charprint(stream, s, '0');
    return;
  }

  if (n<0) {
    charprint(stream, s, '-');
    n *= -1;
  }

  while (n!=0) {
    digit = n%base;
    n /= base;
    
    if (digit<0)
      digit *= -1;

    buf[i++] = digittoascii(digit, upper);
  }

  while (i<min)
    buf[i++] = zpad ? '0' : ' ';

  while (--i>=0)
    charprint(stream, s, buf[i]);
}

static void stringprint(FILE stream, char **s, char *string)
{
  ASSERT(stream == -1 || *s == NULL);
  ASSERT(stream != -1 || *s != NULL);

  if (*s!=NULL) {
    strcpy(*s, string);
    *s += strlen(*s);
  }
  else {
    fputs(string, stream);
  }
}  

static void vgenprintf(FILE stream, char *s, char *format, va_list ap)
{
  int min,zpad;
  char *temp;

  // process format string
  while (*format != 0) {
    // if switch, process
    if (*format == '%') {
      format++;
      // bug: '-' here will potentially screw things up
      ASSERT(*format != '-');

      min=zpad=0;

      if (*format == '0')
	zpad=1;

      min = atoi(format);

      temp = format;
      while (*temp >= '0' && *temp <= '9')
	temp++;

      switch (*(temp++)) {

      case 'c':
	charprint(stream, &s, va_arg(ap, char));
	break;
	
      case 'd':
	integerprint(stream, &s, va_arg(ap, int), 10, min, zpad, 0);
	break;

      case 'x':
	integerprint(stream, &s, va_arg(ap, int), 16, min, zpad, 0);
	break;

      case 'X':
	integerprint(stream, &s, va_arg(ap, int), 16, min, zpad, 1);
	break;

      case 's':
	stringprint(stream, &s, (char*) va_arg(ap, int));
	break;
      
      default:
	charprint(stream, &s, '%');
	temp = format;
      }

      format = temp;
    }
    else
      charprint(stream, &s, *(format++));
  }

  if (s != NULL)
    *s = 0;
}

void vsprintf(char *s, char *format, va_list ap)
{
  vgenprintf(-1, s, format, ap);
}

void vfprintf(FILE stream, char *format, va_list ap)
{
  vgenprintf(stream, NULL, format, ap);
}

void vprintf(char *format, va_list ap)
{
  vfprintf(stdout, format, ap);
}

void sprintf(char *s, char *format, ...)
{
  va_list ap;
  va_start(ap,format);

  vsprintf(s, format, ap);

  va_end(ap);
}

void fprintf(FILE stream, char *format, ...)
{
  va_list ap;
  va_start(ap,format);

  vfprintf(stream, format, ap);

  va_end(ap);
}

void printf(char *format, ...)
{
  va_list ap;
  va_start(ap,format);

  vprintf(format, ap);

  va_end(ap);
}
