// addrspace.h 
//	Data structures to keep track of executing user programs 
//	(address spaces).
//
//	For now, we don't keep any information about address spaces.
//	The user level CPU state is saved and restored in the thread
//	executing the user program (see thread.h).
//
// Copyright (c) 1992-1996 The Regents of the University of California.
// All rights reserved.  See copyright.h for copyright notice and limitation 
// of liability and disclaimer of warranty provisions.

#ifndef ADDRSPACE_H
#define ADDRSPACE_H

#include "copyright.h"
#include "filesys.h"
#include "translate.h"

#define UserStackSize		1024 	// increase this as necessary!

class AddrSpace {
public:
  AddrSpace();				// Create an address space.
  ~AddrSpace();				// De-allocate an address space
  
  // Run the the program stored in the file "executable"
  void Execute(char *fileName);

  void SaveState();			// Save/restore address space-specific
  void RestoreState();			// info on a context switch

  // attempt to extract a string of maximum length "size" (including the null
  // terminator) from the virtual address to the buffer
  // return FALSE if an unrecoverable error occurs or if the string needs more
  // than "size" bytes
  bool ReadString(unsigned vaddr, char* buffer, unsigned size);

  // attempt to copy "size" bytes from the virtual address to the buffer
  // return FALSE if an unrecoverable error occurs
  bool ReadSpace(unsigned vaddr, char* buffer, unsigned size);

  // attempt to copy "size" bytes from the buffer to the virtual address
  // return FALSE if an unrecoverable error occurs
  bool WriteSpace(unsigned vaddr, char* buffer, unsigned size);

private:
  // Assume linear page table translation for now
  TranslationEntry *pageTable;		

  // Number of pages in the virtual address space
  unsigned int numPages;

  // Load the program into memory, return FALSE if an error occurs
  bool Load(char *fileName);

  // Initialize user-level CPU registers, before jumping to user code
  void InitRegisters();		

};

#endif // ADDRSPACE_H
